@@ -14,7 +14,7 @@ use Validation::Class::Fields;
 use Validation::Class::Errors;
 use Validation::Class::Util;
 
-our $VERSION = '7.900018'; # VERSION
+our $VERSION = '7.900019'; # VERSION
 
 use Hash::Flatten 'flatten', 'unflatten';
 use Module::Runtime 'use_module';
@@ -646,10 +646,15 @@ sub get_values {
     my ($self, @fields) = @_;
 
     return () unless @fields;
-
     return (
-        map { $self->fields->has($_) ? $self->fields->get($_)->value : undef }
-        @fields
+        map {
+            my $field = $self->fields->get($_);
+            my $param = $self->params->get($_);
+                $field->readonly ?
+                    $field->default || undef :
+                    $field->value   || $param
+                ;
+        }   @fields
     );
 
 }
@@ -1623,9 +1628,11 @@ sub set_values {
         my $param = $self->params->get($name);
         my $field = $self->fields->get($name);
 
-        next if $field->{readonly} || $field->{default};
+        next if $field->{readonly};
 
-        $self->params->add($name, $value);
+        $value ||= $field->{default};
+
+        $self->params->add($name => $value);
 
         $field->value($value);
 
@@ -1987,7 +1994,7 @@ Validation::Class::Prototype - Data Validation Engine for Validation::Class Clas
 
 =head1 VERSION
 
-version 7.900018
+version 7.900019
 
 =head1 DESCRIPTION
 
@@ -2359,29 +2366,24 @@ no parameter names are passed.
 
 =head2 get_values
 
-The get_values method returns the absolute value (hardcoded, default or
-parameter specified) for a given field. This method executes specific logic
-which returns the value a field has based on a set of internal conditions. This
-method otherwise returns undefined.
+The get_values method returns the absolute value for a given field. This method
+executes specific logic which returns the value a field has based on a set of
+internal conditions. This method always returns a list, field names that do not
+exist are returned as undefined.
 
     my ($value) = $self->get_values('field_name');
 
     # equivalent to
 
-    my $field = $self->fields->get('field_name');
-    my $value = $field->value;
-
-    # equivalent to
-
     my $param = $self->params->get('field_name');
     my $field = $self->fields->get('field_name');
     my $value;
 
-    if ($field->{readonly} || $field->{default}) {
+    if ($field->{readonly}) {
         $value = $field->{default} || undef;
     }
     else {
-        $value = $param;
+        $value = $field->{value} || $param;
     }
 
 =head2 is_valid
